@@ -15,6 +15,8 @@
 
 from searchlight.api import policy
 from searchlight.common import property_utils
+from searchlight.common import resource_types
+from searchlight.common import utils
 from searchlight.elasticsearch.plugins import base
 from searchlight.elasticsearch.plugins.glance \
     import images_notification_handler
@@ -33,11 +35,11 @@ class ImageIndex(base.IndexBase):
             'checksum', 'created_at', 'container_format', 'disk_format', 'id',
             'min_disk', 'min_ram', 'name', 'size', 'virtual_size', 'status',
             'tags', 'updated_at', 'visibility', 'protected', 'owner',
-            'members']
+            'members', 'project_id']
 
     @classmethod
     def get_document_type(cls):
-        return 'OS::Glance::Image'
+        return resource_types.GLANCE_IMAGE
 
     def get_mapping(self):
         return {
@@ -64,51 +66,92 @@ class ImageIndex(base.IndexBase):
                 'min_disk': {'type': 'long'},
                 'min_ram': {'type': 'long'},
                 'owner': {'type': 'string', 'index': 'not_analyzed'},
+                'project_id': {'type': 'string', 'index': 'not_analyzed'},
                 'protected': {'type': 'boolean'},
                 'members': {'type': 'string', 'index': 'not_analyzed'},
                 'created_at': {'type': 'date'},
-                'updated_at': {'type': 'date'}
+                'updated_at': {'type': 'date'},
+
+                # For images based on snapshot
+                'image_type': {'type': 'string', 'index': 'not_analyzed'},
+                'base_image_ref': {'type': 'string', 'index': 'not_analyzed'},
+                'image_state': {'type': 'string', 'index': 'not_analyzed'},
+                'user_id': {'type': 'string', 'index': 'not_analyzed'},
+                'instance_uuid': {'type': 'string', 'index': 'not_analyzed'}
             },
+            "_meta": {
+                "owner": {
+                    "resource_type": resource_types.KEYSTONE_PROJECT
+                },
+                "project_id": {
+                    "resource_type": resource_types.KEYSTONE_PROJECT
+                },
+                "kernel_id": {
+                    "resource_type": resource_types.GLANCE_IMAGE
+                },
+                "ramdisk_id": {
+                    "resource_type": resource_types.GLANCE_IMAGE
+                },
+                "base_image_ref": {
+                    "resource_type": resource_types.GLANCE_IMAGE
+                },
+                "user_id": {
+                    "resource_type": resource_types.KEYSTONE_USER
+                },
+                "instance_uuid": {
+                    "resource_type": resource_types.NOVA_SERVER
+                },
+            }
         }
 
     @property
     def facets_with_options(self):
         return ('disk_format', 'container_format', 'tags', 'visibility',
-                'protected', 'status')
+                'protected', 'status', 'image_type', 'image_state')
+
+    @property
+    def facets_excluded(self):
+        return {'owner': True, 'project_id': True}
+
+    @property
+    def resource_allowed_policy_target(self):
+        return 'get_images'
+
+    @property
+    def service_type(self):
+        return 'image'
 
     def _get_rbac_field_filters(self, request_context):
         return [
-            {
-                'or': [
-                    {
-                        'term': {
-                            'owner': request_context.owner
-                        }
-                    },
-                    {
-                        'term': {
-                            'visibility': 'public'
-                        }
-                    },
-                    {
-                        'term': {
-                            'members': request_context.tenant
-                        }
-                    }
-                ]
-            }
+            {'term': {'owner': request_context.owner}},
+            {'term': {'visibility': 'public'}},
+            {'term': {'members': request_context.tenant}}
         ]
 
     def filter_result(self, hit, request_context):
         super(ImageIndex, self).filter_result(hit, request_context)
+        highlight = hit.get('highlight')
+        if highlight:
+            utils.restore_dots_in_field_names(highlight)
+        source = hit['_source']
+        utils.restore_dots_in_field_names(source)
         if property_utils.is_property_protection_enabled():
-            source = hit['_source']
             for key in list(source.keys()):
                 if key not in self._image_base_properties:
                     if not self.property_rules.check_property_rules(
                             key, 'read', request_context):
                         del source[key]
 
+        # Strip out the members list for non-admins or non-owners. The glance
+        # api leaves just the current tenant in the list for non-admins
+        if not (request_context.is_admin or
+                source['owner'] == request_context.tenant):
+            if source.get('members', []):
+                # Remove any members that don't match the current tenant
+                source['members'] = list(filter(
+                    lambda m: m == request_context.tenant,
+                    source['members']))
+
     def get_objects(self):
         from searchlight.elasticsearch.plugins import openstack_clients
         # Images include their properties and tags. Members are different
